'use strict';

var Promise = require('bluebird');
var lodash = require('lodash');
var util = require('util');
var program = require('commander');
var DevebotApi = require('devebot-api');

var Config = require('../lib/utils/config');
var TextUI = require('../lib/utils/textui');
var Myperf = require('../lib/utils/myperf');
var logger = require('../lib/utils/logger');

var SPECIAL_ARGS = ['package', 'payload'];

module.exports = function (a) {
  a = a || {};

  var b = a.adapter;
  var c = a.listener;
  var d;

  if (lodash.isObject(b)) {
    d = new TextUI();
  } else {
    var e = new Config(a);
    var f = e.getConfig();
    b = new DevebotApi(lodash.extend(f, {
      logger: logger,
      ws: c
    }));
    var g = new Myperf();
    d = new TextUI({ config: e, myperf: g });
  }

  b.on('started', function () {
    logger.debug(' The command is started');
  });

  b.on('progress', function (a) {
    logger.debug(' The command is processing');
  });

  b.on('completed', function (a) {
    d.displayCliOutput(a);
  });

  b.on('failed', function (a) {
    d.displayCliOutput(a);
  });

  b.on('done', function () {
    logger.debug(' - The command is done');
  });

  b.on('noop', function () {
    logger.debug(' - The command not found');
  });

  b.on('close', function (a, b) {
    logger.debug(' - The connection is closed (%s). Message: %s', a, b);
  });

  b.on('error', function (a) {
    logger.debug(' - The connection has been broken - %s', a);
    d.displayException(a);
  });

  var h = Promise.promisify(b.loadDefinition, { context: b });
  h().then(function (a) {
    var c = a && a.payload || {};
    return Promise.resolve().then(function () {
      d.displayCliHeader(c);
      return run(b, c);
    }).then(function () {
      d.displayCliFooter(c);
    });
  }).catch(function (a) {
    d.displayException(a);
  });
};

var run = function run(a, b) {
  return Promise.promisify(function (c) {
    b = b || {};
    logger.trace(' * cli definition: %s', JSON.stringify(b, null, 2));

    var d = b.commands || [];

    for (var e = 0; e < d.length; e++) {
      var f = d[e];
      var g = [];

      var h = program.command(f.name).description(f.description);

      if (f.package) {
        h = h.option('--package [string]', 'Package name');
        g.push('package');
      }

      var i = f.options || [];
      for (var j = 0; j < i.length; j++) {
        var k = i[j];
        h = h.option(util.format('-%s --%s %s', k.abbr, k.name, k.required ? '<value>' : '[value]'), k.description);
        g.push(k.name);
      }

      if (lodash.isObject(f.schema)) {
        h = h.option('--payload [json]', 'Data (in JSON format)');
        g.push('payload');
      }

      h = h.action(function (b, d) {
        return function (e) {
          var f = {
            name: b.name,
            options: lodash.omit(lodash.pick(e, d), SPECIAL_ARGS)
          };
          if (d && d.indexOf('package') >= 0) {
            f['package'] = e['package'];
          }
          if (d && d.indexOf('payload') >= 0) {
            f['payload'] = JSON.parse(e['payload']);
          }
          a.execCommand(f, c);
        };
      }(f, g));
    }

    program.parse(process.argv);

    if (process.argv.length <= 2) {
      program.outputHelp(function (a) {
        process.stdout.write(a);
        c();
      });
    }
  })();
};